
'''
	python3 /biotech/venues/stages/biotech/_status/status.proc.py "_status/monitors/10_time_limits/status_1.py"
'''

import pathlib
from os.path import dirname, join, normpath
import biotech

def check_1 ():
	
	this_directory = pathlib.Path (__file__).parent.resolve ()
	this_module = str (this_directory)
	the_guarantees = str (normpath (join (this_directory, f"guarantees")))
	
	scan = biotech.start (
		glob_string = the_guarantees + "/**/guarantee_*.py",
		
		time_limit = 5,
		
		simultaneous = True,
		simultaneous_capacity = 10,

		module_paths = [
			normpath (join (this_module, "modules")),
			normpath (join (this_module, "modules_pip"))
		],

		relative_path = this_module,
		
		aggregation_format = 2
	)

	assert (scan ["stats"] ["paths"] ["alarms"] == 1), scan ["stats"]
	assert (scan ["stats"] ["paths"] ["empty"] == 0), scan ["stats"]

	assert (scan ["stats"] ["checks"] ["passes"] == 0), scan ["stats"]
	assert (scan ["stats"] ["checks"] ["alarms"] == 0), scan ["stats"]
	
checks = {
	'aggregation format and exit': check_1
}




import json

from flask import Flask, request
import rich

from biotech.procedures.implicit_procedure.process.moves.done_with_scan import done_with_scan_move

def done_with_scan (app):
	@app.route ("/done_with_scan", methods = [ 'PATCH' ])
	def done_patch ():
		print ("@ [patch] /done_with_scan")
	
		the_packet = json.loads (request.data.decode ('utf8'))
		rich.print_json (data = {
			"done_with_scan the_packet": the_packet
		})
		
		done_with_scan_move (
			the_packet = the_packet
		)
	
		return "received"






'''
	stop_process (
		"house_mix.py",
		status = {
			"alarm": "time limit exceeded"
		}
	)
'''

import time

from biotech.procedures.implicit_procedure.process.variables import implicit_procedure_variables

def stop_process (
	the_path,
	status
):
	
	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["status"] ["scan"] = "done"
	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["results_of_scan"] = {
		"path": the_path,
		** status
	}

	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["ended"] = str (time.time ());

	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["elapsed"] = (
		float (implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["ended"]) - 
		float (implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["started"])
	);
	
	#
	#	Once the status of the scan has been established,
	# 	then the scan process can be stopped.
	#
	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["process"] ["process"].terminate ()




'''
	the_scan ["process"].is_alive ()
'''

import time
import traceback

import rich
from biotech.procedures.implicit_procedure.process.variables import implicit_procedure_variables
from biotech.topics.implicit.thread import implicit_thread

from .send_done import send_done
from .aggregate_stats import aggregate_stats
from .stop_process import stop_process

from pprint import pprint

def parse_and_check_is_alive_of_statuses ():
	internal_statuses = implicit_procedure_variables ["internal_statuses"]
	time_limit = implicit_procedure_variables ["intro_variables"] ["time_limit"]

	statuses = {}
	unfinished = []
	for status_path in internal_statuses:
		process_status = "pending"
		
		
		'''
			if over the time limit, stop the process
		'''
		try:
			if (internal_statuses [ status_path ] ["process"] ["process"].is_alive () == True):
				if (time.time () - float (internal_statuses [ status_path ] ["times"] ["started"]) >= float (time_limit)):
					stop_process (
						status_path,
						status = {
							"alarm": "time limit exceeded"
						}
					)
		except Exception as E:
			print ("time limit check exception:", traceback.format_exc ())
			pass;
		
		
		'''
			check if process is_alive is false
		'''
		try:
			#print ("checking process status:", internal_statuses [ status_path ])		
			if (internal_statuses [ status_path ] ["process"] ["process"].is_alive () == False):				
				#
				#	
				#
				#	check if the scan is done, if not the process exitted 
				#	and 
				#
				process_status = "done"
				implicit_procedure_variables ["internal_statuses"] [ status_path ] ["status"] ["process"] = "done"

		except Exception as E:
			print ("parse and check exception:", traceback.format_exc ())
			pass;

		
		except Exception as E:
			pass;
			

		statuses [ status_path ] = {
			"scan": internal_statuses [ status_path ] ["status"] ["scan"],
			"process": process_status
		}
	
	'''
		loop through again,
		because exceptions might
		have occurred.
	'''
	for status_path in statuses:
		if (statuses [ status_path ] ["process"] != "done"):
			unfinished.append ({
				"path": status_path,
				"internals": implicit_procedure_variables ["internal_statuses"] [ status_path ]
			})
		
	return [ statuses, unfinished ]
		

def send_done_if_finished (unfinished):
	if (len (unfinished) >= 1):
		return;
		
	'''
		This loop might be redundant.
	'''
	the_internal_statuses = implicit_procedure_variables ["internal_statuses"]
	for internal_status in the_internal_statuses:
	
		#
		#	if the process is done, then the scan:
		#	
		#		( ) exitted
		#		( ) sent /done_with_scan
		#
		#		( ) unlikely -> neither?
		#
		#if (the_internal_statuses [ internal_status ] [ "status" ] [ "scan" ] != "done"):
		#	return;
	
		if (the_internal_statuses [ internal_status ] [ "status" ] [ "process" ] != "done"):
			return;
	

	'''
		if not bounced, then send done
	'''	
	send_done (
		host = implicit_procedure_variables ["intro_harbor"] ["host"],
		port = implicit_procedure_variables ["intro_harbor"] ["port"],
		
		proceeds = aggregate_stats ()
	)


def print_is_done_status_repetively ():
	details = implicit_procedure_variables ["details"]

	def task (
		stop_event = None
	):		
		while not stop_event.is_set ():
			'''
				check if internal_statuses_built
			'''
			if (implicit_procedure_variables ["internal_statuses_built"] != "yes"):
				continue;
		
			[ internal_statuses, unfinished ] = parse_and_check_is_alive_of_statuses ()
			
			'''
			if (details >= 2):
				pprint ({
						"internal_statuses:": implicit_procedure_variables ["internal_statuses"]
				})
			'''
			
			pprint ({
				"waiting for:": unfinished
			})
			
			send_done_if_finished (unfinished)
			
			time.sleep (1)
			



	the_task = implicit_thread (
		task = task
	)
	the_task ['on'] ()
	
	# the_task ['on'] ()
	




'''
	steps:
		process is started
		
		
'''




#from __keg import open_scan_harbor

from __import_from_path import import_from_path

from __coms.done_with_scan import done_with_scan
from __mixes.format_rel_path import format_rel_path

import os
import time
from pprint import pprint


import sys
for path in sys.path:
	print ("health scan, sys path:", path)

import traceback
import io
def find_trace (exception : Exception) -> str:
	try:
		file = io.StringIO ()
		traceback.print_exception (exception, file = file)
		
		return file.getvalue ().rstrip ().split ("\n")
	except Exception:
		pass;
		
	return 'An exception occurred while calculating trace.'

def main ():
	#raise Exception ("An exception occurred")
	try:
		status_path = os.environ.get ("biotech___status_path")
		status_relative_path = os.environ.get ("biotech___status_relative_path")

		host = os.environ.get ("biotech___harbor_host")
		port = int (os.environ.get ("biotech___harbor_port"))

		proceeds = import_from_path (status_path)

		pprint ({
			"pid": os.getpid (),
			"proceeds": proceeds,
			"harbor": {
				"host": host,
				"port": port
			}
		})
	except Exception as E:
		proceeds = {
			"parsed": False,
			"alarm": "An exception occurred while running the scan.",
			"exception": repr (E),
			"exception trace": find_trace (E)
		}

	done_with_scan (
		host = host,
		port = port,
		
		proceeds = {
			"path": format_rel_path (status_path, status_relative_path),
			"result": proceeds,
			"pid": os.getpid ()
		}
	)

	time.sleep (1)

	exit ()


main ();


#send_post_request (host, port, "/", proceeds)



'''
	from biotech.procedures.implicit_procedure.on import implicit_procedure_on
	implicit_procedure_on ()
'''

'''
	picks:
		( ) sequentially
		( ) simultaneously
		( ) one
'''



'''
	This script starts the keg process.
'''

from botanist.cycle.presents import presents as cycle_presents
import botanist.processes.multiple as multi_proc
import botanist.cycle.loops as cycle_loops
import botanist.ports_v2.available as available_port

from biotech.topics.process_on.p_expect import process_on
from biotech.topics.process_on.p_expect.implicit import process_on_implicit

from .paths import find_implicit_procedure_paths
		
# ----

import pexpect
import rich

# ----

import sys
import json
import os
from fractions import Fraction
import time

# ----

def implicit_procedure_on (
	port,
	packet
):
	limit_start = 25000
		
	path_of_the_scan_process = find_implicit_procedure_paths ()
	process_string = (
		f'''python3 { path_of_the_scan_process } keg open --port { port }'''
	)
	
	process_environment = os.environ.copy ()
	process_environment ["PYTHONPATH"] = ":".join ([
		* sys.path
	])

	the_venture = process_on_implicit (
		process_string,
		
		CWD = None,
		env = process_environment,
		name = "aggregator"
	)
	
	time.sleep (1)
	print ('the implicit procedure:', the_venture)

	return the_venture

	'''
	procs = multi_proc.start (
		processes = [{
			"string": process_string,
			"CWD": None,
			"ENV": process_environment
		}]
	)
	'''

		

	



'''
	from biotech.procedures.intro.keg import open_harbor
	harbor = open_harbor (
		port = 0
	)
	
	harbor.start ()
	harbor.terminate ()
'''


import json
import pathlib
import os
from os.path import dirname, join, normpath
import sys
from multiprocessing import Process
import threading
import time

from flask import Flask, request
import rich


def open_harbor (
	port = 0,
	records = 0,
	
	health_scans_done = lambda *args, **kwargs: None
):
	print ("opening intro harbor on port:", port)

	app = Flask (__name__)

	@app.route ("/", methods = [ 'GET', 'PATCH' ])
	def home_get ():	
		return "?"

	@app.route ("/done", methods = [ 'PATCH' ])
	def done_patch ():
		print ("@ [patch] /done")
	
		the_packet = json.loads (request.data.decode ('utf8'))
		rich.print_json (data = {
			"intro done: the_packet": the_packet
		})
	
		health_scans_done (the_packet)
	
		return "received"

	'''
	
	'''
	def start (stop_event):
		app.run (
			'0.0.0.0',
			port = port,
			debug = False
		)
		
		#return app;
	
	stop_event = threading.Event ()
	
	harbor = threading.Thread (target = start, args=(stop_event, ))
	harbor.daemon = True  # automatically exit when the main program exits
	#harbor.start ()
	
	
	
	# harbor = Process (target = start)
	
	
	'''
		stop_event.set()

		# Wait for the Flask thread to finish
		harbor.join()
	'''
	return [ harbor, stop_event ];
	
	#server.terminate()
	#server.join()



'''

'''

import rich

from biotech.procedures.implicit_procedure.process.variables import implicit_procedure_variables

from .send_done import send_done
from .aggregate_stats import aggregate_stats


def venture_finish ():
	print ()
	print ("checking if is done")
	print ()
	
	the_internal_statuses = implicit_procedure_variables ["internal_statuses"]

	if (implicit_procedure_variables ["internal_statuses_built"] != "yes"):
		return;
	
	for internal_status in the_internal_statuses:
		rich.print_json (
			data = {
				"internal status": {
					internal_status: the_internal_statuses [internal_status] [ "status" ] 
				}
			}
		)
	
	
		#
		#	if the process is done, then the scan:
		#	
		#		( ) exitted
		#		( ) sent /done_with_scan
		#
		#		( ) unlikely -> neither?
		#
		
		#if (the_internal_statuses [ internal_status ] [ "status" ] [ "scan" ] != "done"):
		#	return "no";
			
		if (the_internal_statuses [ internal_status ] [ "status" ] [ "process" ] != "done"):
			return "no";
		
	'''
		if not bounced, then send done
	'''	
	send_done (
		host = implicit_procedure_variables ["intro_harbor"] ["host"],
		port = implicit_procedure_variables ["intro_harbor"] ["port"],
		
		proceeds = aggregate_stats ()
	)



'''
	from biotech.procedures.intro.coms.implicit_procedure.on import await_implicit_procedure_is_on
	await_implicit_procedure_is_on (port = 0)
	
	
	
'''

import botanist.cycle.loops as cycle_loops
from botanist.cycle.presents import presents as cycle_presents

import requests

def await_implicit_procedure_is_on (
	port = ""
):
	print ("awaiting the open of the implicit procedure harbor")

	def send (arg):
		print ('	checking is on')
	
		try:
			URL = f"http://0.0.0.0:{ port }/on"
			response = requests.get (URL)
			if (response.status_code == 200 and response.text == "yes"):
				return True;
		except Exception:
			pass;
			
		raise Exception ('not on')


	the_proceeds = cycle_loops.start (
		send, 
		cycle_presents ([ 1 ]),
		#cycle_presents (),
		
		loops = 20,
		delay = 1,
		
		records = 0
	)
	
	#print ("the_proceeds:", the_proceeds)
	
	return the_proceeds




'''
	import pathlib
	from os.path import dirname, join, normpath
	
	this_folder = pathlib.Path (__file__).parent.resolve ()
	search = normpath (join (this_folder, "../.."))

	import biotech
	biotech.start (
		glob_string = search + '/**/*status.py'
	)
'''



import glob
import json
import pathlib
from os.path import dirname, join, normpath
import os
import time

import rich	
from tinydb import TinyDB, Query

#import biotech.topics.aggregate as aggregate
#import biotech.topics.alarm_parser as alarm_parser

#import biotech.procedures.scan as scan
#from biotech.procedures.scan.on_one import turn_on_one
#from biotech.procedures.scan.on_sequentially import turn_on_sequentially
#from biotech.procedures.scan.on_simultaneously import turn_on_simultaneously

from biotech.topics.printout.passes import printout_passes

# import biotech.procedures.regulators.on as regulators_on

from biotech.procedures.implicit_procedure.on import implicit_procedure_on

from biotech.procedures.intro.keg import open_harbor
from biotech.procedures.intro.coms.implicit_procedure.on import await_implicit_procedure_is_on
from biotech.procedures.intro.coms.implicit_procedure.send_paths import send_paths

import threading
'''
	
'''
def start (
	glob_string = "",
	
	#
	#	itinerary: optionally dynamic
	#
	intro_port = 52434,
	implicit_procedure_port = 52435,
	
	print_alarms = True,
	records = 1,
	db_directory = False,
	
	aggregation_format = 1,
	
	relative_path = False,
	module_paths = [],
	
	simultaneous = False,
	simultaneous_capacity = 10,
	
	time_limit = "99999999999999999999999",
	
	before = False,
	after = False
):
	#regulators_on.on ()
	#return;



	finds = glob.glob (glob_string, recursive = True)
	relative_path = str (relative_path)	
		
	if (records >= 2):
		print ()
		print ("searching for glob_string:")
		print ("	", glob_string)
		print ()
	
	if (records >= 2):
		print ()
		print ("	finds:", finds)
		print ("	finds count:", len (finds))
		print ();

	wait_until_health_scans_done = threading.Event ()

	
	the_scan_results = {}
	
	def health_scans_done (the_packet):
		nonlocal the_scan_results;
		
		the_scan_results = the_packet
	
		#print ("health_scans_done ->", the_packet)
		#print ("health_scans_done:", wait_until_health_scans_done, wait_until_health_scans_done.is_set ())
		
		wait_until_health_scans_done.set ()
		#print ("health_scans_done:", wait_until_health_scans_done, wait_until_health_scans_done.is_set ())

		#print ("await_health_scans_done set")
		#print ()

	[ intro_harbor ] = open_harbor (
		port = intro_port,
		health_scans_done = health_scans_done
	)	
	intro_harbor.start ()

	the_implicit_procedure = implicit_procedure_on (	
		port = implicit_procedure_port,
		packet = {}
	)
	
	#time.sleep (1)
	
	
	'''
		check if the implicit procedure is on
	'''
	await_implicit_procedure_is_on (
		port = implicit_procedure_port
	)
	print ("the implicit procedure has started")
	
	#time.sleep (1)
	
	send_paths (
		port = implicit_procedure_port,
		packet = {
			"status_check_paths": finds,
			
			"relative_path": relative_path,
			"module_paths": module_paths,
			
			"simultaneous": simultaneous,
			"simultaneous_capacity": simultaneous_capacity,
			
			"before": before,
			"after": after,
			
			"aggregation_format": aggregation_format,
			
			"time_limit": time_limit,
			
			"the_intro_harbor": {
				"port": intro_port,
				"host": "0.0.0.0"
			}
		}
	)	
	
	print ()
	print ("paths sent, waiting until the scans are done.")
	print ()
	
	wait_until_health_scans_done.wait ()
	
	print ()
	print ('done awaiting health scans')
	print ()

	intro_harbor.stop ()
	print ('intro harbor is off')

	print ("the_implicit_procedure:", the_implicit_procedure)

	the_implicit_procedure ["process"].terminate ()

	print ("the_implicit_procedure:", the_implicit_procedure)

	rich.print_json (
		data = {
			"intro: the_scan_results": the_scan_results 
		}
	)
	
	if (type (db_directory) == str):
		os.makedirs (db_directory, exist_ok = True)
		db_file = normpath (join (db_directory, f"records.json"))
		db = TinyDB (db_file)
		
		db.insert ({
			'paths': the_scan_results ["paths"], 
			'alarms': the_scan_results ["alarms"],
			'stats': the_scan_results ["stats"]
		})
		
		db.close ()

	return {
		"status": the_scan_results,
		
		"paths": the_scan_results ["paths"],
		"alarms": the_scan_results ["alarms"],
		"stats": the_scan_results ["stats"]
	}

	'''
		This runs the script at the "before" path,
		if the "before" path is a string.
		
		"before" is the same structure as regular checks.
	'''
	if (type (before) == str):
		before_path_statuses = turn_on_one (
			before,
			module_paths,
			relative_path,
			records
		)
		print (
			"before path statuses:", 
			json.dumps (before_path_statuses, indent = 4)
		)
		
		assert (before_path_statuses ['stats']['passes'] >= 1)
		assert (before_path_statuses ['stats']['alarms'] == 0)
		

	'''
		This runs the checks either simultenously or sequentially.
	'''
	if (simultaneous == True):
		path_statuses = turn_on_simultaneously (
			finds,
			module_paths,
			relative_path,
			records,
			
			simultaneous_capacity = simultaneous_capacity,
		)
	else:
		path_statuses = turn_on_sequentially (
			finds,
			module_paths,
			relative_path,
			records
		)
	
	
	'''
		This runs the script at the "after" path,
		if the "after" path is a string.
		
		"after" is the same structure as regular checks.
	'''
	if (type (after) == str):
		after_path_statuses = turn_on_one (
			after,
			module_paths,
			relative_path,
			records
		)
		print ("before path statuses:", json.dumps (after_path_statuses, indent = 4))
		
		assert (after_path_statuses ['stats']['passes'] >= 1)
		assert (after_path_statuses ['stats']['alarms'] == 0)


	'''
		This aggregates (or squeezes) the proceeds of the
		scan into one dictionary (JSON).
	'''
	status = aggregate.start (
		path_statuses,
		
		aggregation_format = aggregation_format
	)
	stats = status ["stats"]
	paths = status ["paths"]
	alarms = alarm_parser.start (status ["paths"])	
		
	
	
	'''
		If a db_directory is designated,
		then this adds the proceeds to the DB.
	'''
	if (type (db_directory) == str):
		os.makedirs (db_directory, exist_ok = True)
		db_file = normpath (join (db_directory, f"records.json"))
		db = TinyDB (db_file)
		
		db.insert ({
			'paths': paths, 
			'alarms': alarms,
			'stats': stats
		})
		
		db.close ()
		
	
	if (records >= 1):
		rich.print_json (data = {
			"paths": paths
		})
		
		printout_passes (paths)
		
		rich.print_json (data = {
			"alarms": alarms
		})
		rich.print_json (data = {
			"stats": stats
		})		
		
	return {
		"status": status,
		
		"paths": paths,
		"alarms": alarms,
		"stats": stats
	}
	





'''

'''

from biotech.procedures.implicit_procedure.process.moves.format_path import format_path


'''
	paths_statuses = [{
		"path": path,
		** scan_status
	}]
'''

'''
	the_internal_statuses = {
		"status_2.py": {
			"status": {
				"scan": "pending",
				"process": "pending",
			},
			
			
			
			#
			#	"pending"
			#
			"scan status": "done",
						
			"records": [],
			"venture": process_on
		}
	}
'''
implicit_procedure_variables = {
	"intro_harbor": {
		"host": "0.0.0.0",
		"port": ""
	},
	
	"intro_variables": {},
	
	#
	#	This is the list of statuses
	#
	#"paths_statuses": [],
	
	"details": 2,
	
	#
	#
	#
	"internal_statuses": {},
	"internal_statuses_built": "no"
}

def setup_internal_statuses (
	status_check_paths,
	relative_path
):
	for status_check_path in status_check_paths:		
		implicit_procedure_variables ["internal_statuses"] [ format_path (status_check_path, relative_path) ] = {
			"status": {
				"scan": "pending",
				"process": "pending"
			},
			
			"times": {
				"started": "",
				"ended": "",
				"elapsed": ""
			},
			
			"process": None,
			"results_of_scan": None
		}
			
			
	return;

def change ():
	return;
	
def retrieve ():
	return implicit_procedure_variables;




print ('implicit procedure :: process')

import rich
import sys
rich.print_json (data = {
	"implicit procedure sys paths": sys.path
})


from clique import start_clique
from biotech.procedures.implicit_procedure.process.moves.print_is_done_status_repetively import print_is_done_status_repetively

print_is_done_status_repetively ()

start_clique ()



import json
import pathlib
import os
from os.path import dirname, join, normpath
import sys
import threading
import time


from flask import Flask, request
import rich

from .spaces.done_with_scan import done_with_scan
from .spaces.paths_patch import paths_patch



def open_harbor (
	port = 0,
	records = 0
):
	if (records >= 1):
		print ("opening scan process keg on port:", port)

	#app = Flask (__name__)
	#app = Flask ("implicit procedure harbor")
	app = Flask ("aggregator")


	@app.route ("/", methods = [ 'GET' ])
	def home_get ():	
		return "received"


	@app.route ("/on", methods = [ 'GET' ])
	def on_get ():	
		return "yes"

	
	done_with_scan (app)
	paths_patch (
		app,
		
		implicit_procedure_port = port
	)



	app.run (
		'0.0.0.0',
		port = port,
		debug = False
	)





import json
import pathlib
import os
from os.path import dirname, join, normpath
import sys
import threading
import time


from flask import Flask, request
import rich

from biotech.topics.queues.queue_capacity_limiter import queue_capacity_limiter
from biotech.procedures.health_scan.on import turn_on_health_check

from biotech.procedures.implicit_procedure.process.variables import implicit_procedure_variables, setup_internal_statuses
from biotech.procedures.implicit_procedure.process.moves.format_path import format_path



def paths_patch (
	app,
	
	implicit_procedure_port = None
):
	@app.route ("/paths", methods = [ 'PATCH' ])
	def paths_patch ():
		print ("@ [patch] /paths")
	
	
		'''
			{
				"paths": [],

				"relative_path": False,
				"relative_path": "/biotech/venues/warehouse/0_example/modules",
				
				module_paths = [],
				
				simultaneous = False,
				simultaneous_capacity = 10,
				
				before = False,
				after = False
			}
		'''	
		the_packet = json.loads (request.data.decode ('utf8'))
		rich.print_json (data = {
			"paths the_packet": the_packet
		})

		#----
		#
		status_check_paths = the_packet ["status_check_paths"]

		module_paths = the_packet ["module_paths"]
		relative_path = the_packet ["relative_path"]

		simultaneous = the_packet ["simultaneous"]
		simultaneous_capacity = the_packet ["simultaneous_capacity"]
		
		before = the_packet ["before"]
		after = the_packet ["after"]
		#
		#----
		
		implicit_procedure_variables ["intro_variables"] = the_packet
		implicit_procedure_variables ["intro_harbor"] = the_packet ["the_intro_harbor"]		
		
		
		
		'''
			This initializes the internal statuses.
		'''
		setup_internal_statuses (
			status_check_paths,
			relative_path
		)
		implicit_procedure_variables ["internal_statuses_built"] = "yes"
		#
		# ----
		
		
		
		def venture (status_check_path):
			start_time = str (time.time ())
		
			the_scan = turn_on_health_check (
				packet = {
					"status_check_path": status_check_path,
					
					"module_paths": module_paths,
					"relative_path": relative_path,
					
					"implicit_procedure": {
						"port": implicit_procedure_port
					}
				}
			)
			
			print ()
			print ("the_scan:", the_scan)
			print ()
			
			#the_scan_processes [ status_check_path ] = the_scan
			implicit_procedure_variables [
				"internal_statuses"
			] [ 
				format_path (status_check_path, relative_path) 
			] [ 
				"process" 
			] = the_scan
		
		
			implicit_procedure_variables [
				"internal_statuses"
			] [ 
				format_path (status_check_path, relative_path) 
			] [
				"times"
			] [
				"started"
			] = start_time
		
			return the_scan;
		
		
		
		if (simultaneous):
			proceeds = queue_capacity_limiter (
				capacity = simultaneous_capacity,
				items = status_check_paths,
				move = venture
			)		
			
			print ("queue_capacity_limiter proceeds:", proceeds)
		
		else:
			for status_check_path in status_check_paths:
				venture (status_check_path)
		
		'''
			concurrency
		'''
	
		return "received"




'''
	This turns off the scan process.
'''

import json
import pathlib
import os
from os.path import dirname, join, normpath
import sys
import threading
import time

from flask import Flask, request

import rich


from biotech.procedures.implicit_procedure.process.variables import implicit_procedure_variables

from .aggregate_stats import aggregate_stats



def done_with_scan_move (
	the_packet = ""
):
	the_path = the_packet ["path"]
	the_result = the_packet ["result"]
	the_pid = the_packet ["pid"]
	
	'''
		turn off the scan process
	'''
	#print ('turning off scan process with pid:', the_pid)
	#os.kill (the_pid, 9)
	
	#
	#	stopping
	#
	print ("stopping:", implicit_procedure_variables ["internal_statuses"] [ the_path ] ["process"] ["process"])
	
	
	status = {
		"path": the_path,
		** the_result
	}
	#implicit_procedure_variables ["paths_statuses"].append (status)
	
	
	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["status"] ["scan"] = "done"
	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["results_of_scan"] = status

	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["ended"] = str (time.time ());

	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["elapsed"] = (
		float (implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["ended"]) - 
		float (implicit_procedure_variables ["internal_statuses"] [ the_path ] ["times"] ["started"])
	);
	
	#
	#	Once the status of the scan has been established,
	# 	then the scan process can be stopped.
	#
	implicit_procedure_variables ["internal_statuses"] [ the_path ] ["process"] ["process"].terminate ()
	
	



'''
	from keg.send.done import send_done
	send_done ()
'''

import requests


def send_done (
	host = "0.0.0.0",
	URL_path = "/done",
	port = "",
	
	proceeds = {}
):
	URL = f"http://{ host }:{ port }{ URL_path }"
	print ("URL:", URL)

	response = requests.patch (URL, json = proceeds)

	print ("Response status code:", response.status_code)
	print ("Response content:", response.text)